
# dmix: Using a bluealsa playback device with multiple applications

__*Important note: this solution requires features of bluealsa introduced in release 3.0.0. It is not suitable for earlier releases (release 2.1.0 or earlier); those versions are not compatible with `alsaloop`.*__

## Introduction

A bluealsa PCM can only be opened by one process at a time. This can be inconvenient in many scenarios. For example, many audio applications keep the alsa PCM device open, even when they are not actively streaming any audio, and so it is necessary to close each application before any other can use the device; desktop systems may like to emit system sounds while music is playing; etc.

ALSA provides a solution to this for hardware cards with the "dmix" plugin. That plugin allows multiple open connections, mixes the streams together, then sends the result as a single stream to the hardware device. Unfortunately dmix works as a front-end only with hardware (ie "hw" type) devices, and not with any other plugin types such as bluealsa.

So it is not possible to configure bluealsa as a backend device for dmix. We are forced to find an indirect method; that is, use a "hw" device that can forward the mixed stream on to a bluealsa PCM. Fortunately, ALSA provides just such a device, the "Loopback" device, which is a kernel driver that implements a virtual sound card.

This wiki article describes one way that bluealsa can be used with the ALSA Loopback device and the dmix plugin to achieve mixing of multiple audio streams into a single bluealsa playback PCM.

## Overview

In this solution, we use ALSA's `dmix` plugin to mix the streams, ALSA's `Loopback` kernel module to provide the hardware sound card interface required by dmix, and ALSA's `alsaloop` utility to transfer the mixed stream from the Loopback device to a bluealsa PCM device. As you will see, this is cumbersome to achieve manually, but fortunately much of the work can be scripted. An example scripted solution is given at the end of this article.

### snd-aloop

The Loopback device is created by a kernel module called `snd-aloop`. This module is part of the upstream Linux kernel sources, and the majority of distributions now include it by default. To use it you have to load it, for example with:
```
sudo modprobe snd-aloop
```
Once loaded, the module creates a virtual soundcard called `Loopback`. This card provides two PCM devices, one for capture and one for playback. One application plays an audio stream into device 0, and another captures that same stream from device 1. To illustrate:
```
aplay -D hw:Loopback,0 music.wav
arecord -D hw:Loopback,1 captured.wav
```
The card does not perform any format conversions or resampling, so the two applications must use the same sample format, channel count, and sample rate. The card permits a range of formats, to see which are available, try:
```
aplay -D hw:Loopback --dump-hw-params /dev/zero
```
The Loopback device will adopt the configuration negotiated with the first application to connect, which can be either the playback or the capture device, and the second application must accept that configuration. For that reason, it is common to use Loopback in combination with the `plug` plugin on the second connection. So for example, format conversion can be achieved if we connect the capture side first with the desired output format, then connect the playback side through plug:
```
arecord -f float_le -c 2 -r 8000 -D hw:Loopback,1 capture.wav &
aplay -D plughw:Loopback,0 /usr/share/sounds/alsa/Front_Center.wav
```
The source file is mono, S16_LE, sampled at 48000 Hz; the captured file will be stereo FLOAT_LE sampled at 8000Hz.

[ Note that you will have to kill the arecord process after running the example above, because the Loopback will not close its output device when the input application disconnects. ]

The Loopback device actually supports multiple "substreams": the previous examples used the default substream, number 0. The `snd-aloop` module produces 8 substreams by default, which is the maximum supported, but that number can be reduced if required by passing a parameter when the module is loaded. For example use only 2 substreams, load the module as:

`sudo modprobe snd-aloop pcm_substreams=2`

Each output substream is paired with the corresponding input substream. So, for example, `hw:Loopback,0,2` will output the stream that is being sent to `hw:Loopback,1,2`. As noted above, if the substream is not specified it defaults to 0.

### alsaloop

The second piece of the puzzle is `alsaloop`. Just as Loopback is a device that connects two applications, so alsaloop is an application that connects two devices. It is a standard alsa utility, along with aplay and arecord, and is included by most Linux distributions. We will use it to capture a stream from the Loopback output device and send it to a bluealsa PCM device.

alsaloop will try to manage the end-to-end latency between the devices, and we need to choose the target latency carefully to avoid underruns. It also allows the user to specify a desired audio format for the loop, which we can use to avoid unnecessary format conversions.

alsaloop will also attempt to compensate for different clocks in the capture and playback devices by modifying the audio stream in various ways according to the `--sync` argument. It is recommended to use `--sync=none` in this use case and rely on the buffering within bluealsa to smooth out minor sync variations. So a simple usage scenario setting the latency to 50 milliseconds would be:

```
alsaloop -C hw:Loopback,1,0 -P bluealsa:DEV=XX:XX:XX:XX:XX:XX,PROFILE=a2dp --sync=none -c 2 -r 48000 -f s16_le -t 50000
```
If this is run before an application connects to the Loopback input device, the Loopback will be configured as stereo S16_LE format with a sample rate of 48000Hz. This avoids any conversion by the bluealsa device.

Now an application that wishes to playback to the bluealsa device can do so as:
```
aplay -D plughw:Loopback,0,0 music.wav
```

For more information on alsaloop command line options, consult the man page
```
man alsaloop
```
#### A note about latency and the bluealsa DELAY parameter

alsaloop tries hard to achieve the target latency set with the ` -t ` parameter. It uses the delay values reported by the capture and playback devices to calculate its own internal settings dynamically. If it is set an impossible target this can lead to constant underruns, and sometimes alsaloop may fail completely.

For this reason it is advisable to set the bluealsa DELAY parameter to 0 when using with alsaloop so that bluealsa understates, rather than overstating, the actual playback delay.

If the latency is not important to your application (within reason) then use ` alsaloop -t 500000 `; if latency is an issue (e.g. with VLC etc) then try ` alsaloop -t 20000`, and increase the value if underruns occur.

### dmix

The final puzzle piece is `dmix`. By wrapping the Loopback input device with dmix, we can allow multiple applications to send audio to the bluealsa PCM device simultaneously. To achieve this, we need to write an ALSA configuration entry. Possibly the simplest example is:
```
pcm.loopback0 {
    type plug
    slave {
        pcm {
            type dmix
            ipc_key 1024
            slave {
                pcm {
                    type hw
                    card "Loopback"
                    device 0
                    subdevice 0
                }
            }
        }
    }
}
```
That creates a PCM called "loopback0" which wraps `hw:Loopback,0,0`. If used in combination with the alsaloop example above, then the sound will play on the bluetooth device XX:XX:XX:XX:XX:XX. To use it, copy the above configuration entry to your .asoundrc file, then:

0. load the snd-aloop module if not already done

   `sudo modprobe snd-aloop`

1. connect the bluetooth speaker

2. run the alsaloop command

   `alsaloop -C hw:Loopback,1,0 -P bluealsa:DEV=XX:XX:XX:XX:XX:XX,PROFILE=a2dp -c2 -r48000 -fs16_le -t 50000`

3. play the music

   `aplay -D loopback0 music.wav`

The above example alsa configuration is very basic; for more information on the dmix plugin, and other official ALSA plugins, see here: https://www.alsa-project.org/alsa-doc/alsa-lib/pcm_plugins.html


## Limitations

### Device disconnection

When a bluetooth device is disconnected, it is normally expected that an application playing to it will be informed and respond appropriately. Unfortunately that is not the case with this solution. The Loopback devices remain active as long as the `snd-aloop` kernel module is loaded. So a playback application will continue to play, even though there is no speaker to render the sound. There does not appear to be any workaround for this issue.

### Maximum 8 pcms

The alsa Loopback device imposes a maximum of 8 substreams, which in turn means that we can have a maximum of 8 pcm devices connected at any one time. Note that a bluetooth device that offers both A2DP and HFP/HSP profiles will appear as 2 distinct bluealsa pcms, so will use 2 Loopback substreams. In most situations, this limit is sufficient - (typically only one or two bluetooth playback devices will be in use at any one time). However, if paired devices auto-connect and there are many in the local bluetooth environment, then this limit can become a problem.

### Synchronization

The alsa Loopback device does not propagate the delay (latency) reporting from the output device to the input device. So applications will see only the (negligible) delay within the snd-aloop module, and not the (considerable) delay with the bluetooth system. This means that applications that try to dynamically synchronize audio playback (with video etc) will not be able to do so.

### Complexity

This solution is complex; setting up the dmix wrapper requires writing alsa configuration entries and identifying the correct arguments with which to start alsaloop. If only one or two bluetooth playback devices are used, then it is feasible to write the alsa configuration into `~/.asoundrc` and to start alsaloop manually when required. But when multiple bluetooth playback devices are in use this becomes onerous and error-prone. It is possible to automate much of the setup, and one possible approach is described in the next section.

---

## Automation (bmix)

This section describes one possible method for automating the set-up of dmix wrappers for bluealsa pcms. We call this solution `bmix`. In order to guarantee that a Loopback substream is always allocated to the same bluealsa pcm, this solution places a hard limit of 8 playback pcms that can be configured.

### Objectives

- monitor bluetooth playback device connection and disconnection;
- launch an instance of alsaloop for each new device connection;
- dynamically create an alsa pcm configuration node for the appropriate loopback substream, using the same name as the control shown by amixer;
- remove the configuration node and terminate alsaloop when the device disconnects;
- a particular device always uses the same Loopback substream so that applications that cache the alsa configuration are not confused when a device disconnects then re-connects.

### Components

- service account - for managing permissions;
- static alsa configuration file - pcm definition templates;
- dynamic alsa configuration file - pcm definitions for dmix wrappers;
- dbus configuration - grant permission to interact with dbus services;
- daemon - script to monitor dbus, manage alsaloop and generate alsa configuration;
- systemd unit - for integration into system management;
- modules unit - loads the Loopback module at boot.

#### Service account

To avoid the need to run any services as `root`, create a service account `bmix` specifically for use by the bmix system. This account should have its own group `bmix` for file ownership, and should also be a member of the `audio` group.

Also create a data directory for use by this account:

`drwxr-xr-x 2 bmix bmix 4096 Jun 15 18:41 /var/lib/bmix/`

#### Static alsa configuration file

To simplify the task of specifying the configuration of a dmix wrapper for bluealsa, and to guard against unnecessary audio format conversions, we install a file into the alsa configuration directory to define parameterized pcm nodes.

For alsa versions 1.1.7 and later, this file should be saved as:

`/etc/alsa/conf.d/21-bmix.conf`

and for alsa versions 1.1.6 and earlier:

`/usr/share/alsa/alsa.conf.d/21-bmix.conf`


The contents of this file is :

```
pcm.bluealsa_raw {
	@args [ DEV PROFILE DELAY SRV ]
	@args.DEV {
		type string
		default {
			@func refer
			name defaults.bluealsa.device
		}
	}
	@args.PROFILE {
		type string
		default {
			@func refer
			name defaults.bluealsa.profile
		}
	}
	@args.DELAY {
		type integer
		default {
			@func refer
			name defaults.bluealsa.delay
		}
	}
	@args.SRV {
		type string
		default {
			@func refer
			name defaults.bluealsa.service
		}
	}
	type bluealsa
	device $DEV
	profile $PROFILE
	delay $DELAY
	service $SRV
}

defaults.bmix {
	loop 0
	channels 2
	rate 48000
	period 20000
}

pcm.bmix {
	@args [ IPC_KEY LOOP CHANNELS RATE PERIOD ]
	@args.IPC_KEY {
		type integer
	}
	@args.LOOP {
		type integer
		default {
			@func refer
			name defaults.bmix.loop
		}
	}
	@args.CHANNELS {
		type integer
		default {
			@func refer
			name defaults.bmix.channels
		}
	}
	@args.RATE {
		type integer
		default {
			@func refer
			name defaults.bmix.rate
		}
	}
	@args.PERIOD {
		type integer
		default {
			@func refer
			name defaults.bmix.period
		}
	}
	type plug
	slave {
		pcm {
			type dmix
			ipc_key $IPC_KEY
			ipc_perm 0660
			ipc_gid audio
			slave {
				channels $CHANNELS
				pcm {
					type hw
					card "Loopback"
					device 0
					subdevice $LOOP
					format "s16_le"
					rate $RATE
				}
				period_time $PERIOD
			}
		}
	}
	hint {
		show {
			@func refer
			name defaults.namehint.basic
		}
		description "Bluetooth Audio - Mix Multiple Streams"
	}
}

ctl.bmix {
	@args [ BAT SRV ]
	@args.BAT {
		type string
		default {
			@func refer
			name defaults.bluealsa.battery
		}
	}
	@args.SRV {
		type string
		default {
			@func refer
			name defaults.bluealsa.service
		}
	}
	type bluealsa
	battery $BAT
	service $SRV
}

</var/lib/bmix/bmix.conf>
```

**Download the file [21-bmix.conf](files/Using-bluealsa-with-dmix/21-bmix.conf)**

This file should be owned by root, with permissions `644` (ie readable by all users, writable only by root)

#### Dynamic alsa configuration file

We also need an alsa configuration node for each specific bluetooth pcm that is connected. These entries will be added and removed as devices connect and disconnect, so the file needs to be edited by the  bmix system. This file is included by the static configuration file above, and so it _**must exist**_, even if empty, otherwise alsa applications will report a configuration error and fail.

The file should be owned by `bmix`, and be readable by everyone:

`-rw-r--r-- 1 bmix bmix 62 Jun 15 18:41 /var/lib/bmix/bmix.conf`

Initially it should be empty, the bmix system will manage its contents.

#### D-Bus configuration file

The bmix daemon needs to exchange messages via d-bus to both `org.bluealsa` and `org.bluez`. Membership of the `audio` group permits the `org.bluealsa` communication, but to permit the `org.bluez` messages we create a new D-Bus configuration file, saved as:

`/etc/dbus-1/system.d/bmix.conf`

```
<!-- This configuration file specifies the required security policies
     for bmix bluealsa helper daemon to work. -->

<!DOCTYPE busconfig PUBLIC "-//freedesktop//DTD D-BUS Bus Configuration 1.0//EN"
 "http://www.freedesktop.org/standards/dbus/1.0/busconfig.dtd">
<busconfig>

  <!-- ../system.conf have denied everything, so we just punch some holes -->

  <policy user="bmix">
    <allow send_destination="org.bluez"/>
  </policy>

</busconfig>
```

**Download the file [bmix.conf](files/Using-bluealsa-with-dmix/bmix.conf)**

D-Bus will need to re-load its configuration before this new policy will become effective. For systems using systemd, this can be achieved by:

`sudo systemctl reload dbus.service`

#### Daemon (monitor script)

This script is the active component of the bmix system, it detects when a bluetooth audio device connects or disconnects, and in response manages the necessary `alsaloop` processes and edits the dynamic alsa configuration. It should be installed as:

`/usr/local/bin/bmixd.bash`

Ownership and permissions:

`-rwxr-xr-x 1 root root 6067 Jun  4 16:49 /usr/local/bin/bmixd.bash`

```
#!/bin/bash

# revision 0.2


# select which profile types to use: "a2dp", "sco", or "all"
profiles="${BMIX_PROFILE:-all}"

# d-bus service name for bluealsa
servicename="${BMIX_BLUEALSA_SRV:-org.bluealsa}"

# experiment with higher latency if you experience underruns/overruns
latency="${BMIX_LATENCY:-50000}"

# location of dynamic alsa configuration file
conffile="${BMIX_ALSA_CONF:-/var/lib/bmix/bmix.conf}"

# dmix ipc will use 8 consecutive ipc key numbers starting with this value
ipc_key_start=${BMIX_IPC_KEY:-30000}

# to reserve some Loopback substreams for other applications, set the lowest
# substream to use here:
lowest_loop_substream=${BMIX_LOOPBACK:-0}

# name to use for default bluetooth pcm
default_pcm_name="${BMIX_DEFAULT_PCM_NAME:-bluetooth}"

# extra arguments to apply to alsaloop
alsaloop_args="${BMIX_ALSALOOP_ARGS:---sync=none}"


ALSALOOP="alsaloop $alsaloop_args"


awk_parse_dbus_message='
	/member=PCMAdded/   { added = 1; next; }
	/member=PCMRemoved/ { removed = 1; next; }
	/member=NameOwnerChanged/ { printf("ServiceStopped\n"); fflush(); next; }
	!/variant/ && /object path/ {
		match($0, /dev_([[:xdigit:]_]+)\/(a2dp|hfp|hsp)(.*)/, arr)
		if (RSTART > 0) {
			brackets = 0
			addr = arr[1]
			if (removed) {
				if (arr[2] ~ /hfp|hsp/)
					profile = "SCO"
				else if (arr[2] ~ a2dp)
					profile = "A2DP"
				else
					next
				if (arr[3] ~ /source/)
					printf("PCMRemoved %s %s %s\n", addr, profile, "source")
				else if (arr[3] ~ /sink/)
					printf("PCMRemoved %s %s %s\n", addr, profile, "sink")
				else
					next
				fflush()
				removed = 0
				next
			}
			profile = ""
			mode = ""
			format = ""
			channels = ""
			sampling = ""
			next
		}
	}
	/string "Transport"/ { t = 1; next; }
	/string "Mode/       { m = 1; next; }
	/string "Format"/    { f = 1; next; }
	/string "Channels"/  { c = 1; next; }
	/string "Sampling"/  { s = 1; next; }
	t && /A2DP/    { profile = "A2DP"; t = 0; next; }
	t && /HFP|HSP/ { profile = "SCO"; t = 0; next; }
	m && /string/ {
		sub(/^.*string /,"")
		gsub("\"","")
		mode = $0
		next
	}
	f && /uint16/ { format = $NF; f = 0; next; }
	c && /byte/   { channels = $NF; c = 0; next; }
	s && /uint32/ { sampling = $NF; s = 0; next; }
	m && /)/ { m = 0; next; }
	t && /)/ { t = 0; next; }
	f && /)/ { f = 0; next; }
	c && /)/ { c = 0; next; }
	s && /)/ { s = 0; next; }
	/\[/ { brackets++; next; }
	addr && /]/ && (--brackets == 0) {
		if (addr && profile && mode && format && channels && sampling) {
			if (added)
				printf("PCMAdded ")
			printf("%s %s %s %s %s %s\n", addr, profile, mode, format, channels, sampling)
			fflush()
			added = 0
		}
	}
'

declare -A formats
formats[264]=U8
formats[33296]=S16_LE
formats[33560]=S24_3LE
formats[33816]=S24_LE
formats[33824]=S32_LE
formats[8]=U8            # pre bluealsa v3.0.0
formats[32784]=S16_LE    # pre bluealsa v3.0.0
formats[32792]=S24_LE    # pre bluealsa v3.0.0

declare -A select_profile
select_profile[A2DP]=yes
select_profile[SCO]=yes

profiles=${profiles^^?}
if [[ "$profiles" = SCO ]] ; then
	select_profile[A2DP]=no
elif [[ "$profiles" = A2DP ]] ; then
	select_profile[SCO]=no
fi

declare -A pids
declare -A loops
declare -A name

declare default_devid=""

declare -i first_loop="$lowest_loop_substream"
declare -i num_loops=$(grep -c substream /proc/asound/Loopback/cable\#0)

get_loop_by_devid() {
	declare -n result=$1
	declare devid="$2"
	declare i
	result=""
	for i in "${!loops[@]}" ; do
		if [[ "$i" = "$devid" ]] ; then
			result=${loops[$i]}
			break
		fi
	done
	if [[ -z "$result" ]] ; then
		declare -i next_loop=$((first_loop + ${#loops[@]}))
		if [[ $next_loop -lt $num_loops ]] ; then
			loops["$devid"]=$next_loop
			result=$next_loop
		fi
	fi
}

launch_alsaloop() {
	# arg1 devid
	# arg2 loop
	# arg3 addr
	# arg4 profile
	# arg5 format
	# arg6 channels
	# arg7 sample rate
	declare devid="$1" loop=$2 addr="$3" profile="$4" format="$5" channels=$6 rate=$7 pid

	$ALSALOOP -f "$format" -c "$channels" -r "$rate" -C hw:Loopback,1,$loop -P bluealsa_raw:SRV="$servicename",DEV="$addr",PROFILE="$profile",DELAY=0 -t "$latency" >/dev/null 2>&1 &
	pid=$!
	sleep 1
	if ! kill -0 $pid >/dev/null 2>&1; then
		echo "failed to start alsaloop for $addr ($profile)" >&2
		return 1
	fi
	pids["$devid"]=$pid
}

create_alsa_config() {
	# arg1 device id
	# arg2 device alias
	# arg3 profile
	# arg4 channels
	# arg5 rate
	declare devid="$1" dev_alias="$2" profile="$3"
	declare -i channels=$4 rate=$5
	declare -i ipc_key=$((ipc_key_start + loops[$devid]))
	declare loop

	cat >> "$conffile" <<-EOF
	pcm."${name[$devid]}".type empty
	pcm."${name[$devid]}".slave.pcm "bmix:IPC_KEY=${ipc_key},LOOP=${loops[$devid]},CHANNELS=${channels},RATE=${rate},PERIOD=$(($latency / 2))"
	pcm."${name[$devid]}".hint.show.@func refer
	pcm."${name[$devid]}".hint.show.name defaults.namehint.basic
	pcm."${name[$devid]}".hint.description "$dev_alias ($profile) Bluetooth Audio Playback"
	EOF
}

# get device alias from Bluez
get_alias() {
	# arg1 name of variable to store result
	# arg2 addr (underscored)
	declare -n result="$1"
	result=$(dbus-send --print-reply=literal --system --dest=org.bluez \
	    /org/bluez/hci0/dev_$2 \
	    org.freedesktop.DBus.Properties.Get string:"org.bluez.Device1" string:"Alias")
	result=${result#   variant       }
}

add_pcm() {
	# arg1 addr (underscored)
	# arg2 profile
	# arg3 format
	# arg4 channels
	# arg5 sample rate
	declare devid="$1,$2" profile="$2" format=${formats["$3"]} addr="${1//_/:}"
	declare -i channels=$4 rate=$5
	declare loop dev_alias

	get_alias dev_alias "$1"
	[[ -n "$dev_alias" ]] || return 1

	get_loop_by_devid loop $devid

	if [[ -z "$loop" ]] ; then
		echo "No free Loopback substreams" >&2
		return 1
	fi
	launch_alsaloop "$devid" "$loop" "$addr" "$profile" "$format" "$channels" "$rate" \
	  || return 1
	name["$devid"]="$dev_alias - $profile"
	create_alsa_config "$devid" "$dev_alias" "$profile" "$channels" "$rate"
}

remove_pcm() {
	#arg1 devid
	declare devid="$1"
	sed -i '/^pcm."'"${name["$devid"]}"'/d' "$conffile"
	kill "${pids["$devid"]}" 2>/dev/null
	unset pids["$devid"]
	unset name["$devid"]
}

# make sure default name is alias for pcm that is currently connected
update_default() {
	for devid in "${!name[@]}" ; do
		[[ "$devid" = "$default_devid" ]] && return
	done
	declare -a arr=("${!name[@]}")
	default_devid="${arr[0]}"
	sed -i -e '/^pcm."'"$default_pcm_name"'/d' "$conffile"	2>/dev/null
	if [[ -n "$default_devid" ]] ; then
		cat >> "$conffile" <<-EOF
		pcm."$default_pcm_name".type empty
		pcm."$default_pcm_name".slave.pcm "${name[$default_devid]}"
		pcm."$default_pcm_name".hint.show.@func refer
		pcm."$default_pcm_name".hint.show.name defaults.namehint.basic
		pcm."$default_pcm_name".hint.description "Default bluetooth playback"
		EOF
	fi
}

# get list of connected devices
get_devices() {
	dbus-send --print-reply --system --dest=org.bluealsa \
		/org/bluealsa org.bluealsa.Manager1.GetPCMs 2>/dev/null | gawk "$awk_parse_dbus_message"
}

handle_device_added_event() {
	if [[ "$4" = sink && "${select_profile["$3"]}" = yes ]] ; then
		add_pcm "$2" "$3" "$5" "$6" "$7"
		update_default
	fi
}

handle_device_removed_event() {
	if [[ "$4" = sink ]] ; then
		remove_pcm "$2,$3"
		update_default
	fi
}

add_initial_devices() {
	readarray -t devices < <(get_devices)
	for device in "${devices[@]}" ; do
		params=($device)
		if [[ "${params[2]}" = sink && "${select_profile["${params[1]}"]}" = yes ]] ; then
			add_pcm "${params[0]}" "${params[1]}" "${params[3]}" "${params[4]}" "${params[5]}"
		fi
	done
	update_default
}

# remove all pcms if bluealsa service terminates
handle_service_stopped_event() {
	for pcm in "${!name[@]}" ; do
		remove_pcm "$pcm"
	done
	update_default
}

# delete all stale entries from alsa config file
echo "# DO NOT DELETE, DO NOT EDIT - automatically managed by bmixd" > "$conffile"

# create a temporary named pipe to communicate with dbus monitor
PIPE=$(mktemp -u)
mkfifo $PIPE
# attach it to unused file descriptor FD
exec {FD}<>$PIPE
# unlink the named pipe
rm $PIPE

# make sure the pipeline is shut down if this script interrupted
trap "kill %1; exec {FD}>&-; handle_service_stopped_event; exit 0" INT TERM

# start dbus monitor in background
dbus-monitor --system "type='signal',sender='org.bluealsa',interface='org.bluealsa.Manager1'" "sender='org.freedesktop.DBus',member='NameOwnerChanged',arg0='org.bluealsa',arg2=''" 2>/dev/null | gawk "$awk_parse_dbus_message" >&$FD &

# load initial set of connected devices
add_initial_devices

# now listen for dbus signals
while read
do
	case "$REPLY" in
		PCMAdded*)
			handle_device_added_event $REPLY
			;;
		PCMRemoved*)
			handle_device_removed_event $REPLY
			;;
		ServiceStopped*)
			handle_service_stopped_event
			;;
	esac
done  <&$FD
```

**Download the file [bmixd.bash](files/Using-bluealsa-with-dmix/bmixd.bash)**

The daemon should _*not*_ be run as root, as that might present a security risk. Run it as user "bmix" in group "audio" as described in the "Service account" section above.

There are no command-line arguments, but some properties can be modified by setting evironment variables:

- **BMIX_PROFILE** \
  select which profile types to use: "a2dp", "sco", or "all" \
  *default* "all"

- **BMIX_BLUEALSA_SRV** \
  d-bus name of the bluealsa service \
  *default* "org.bluealsa"

- **BMIX_LATENCY** \
  experiment with higher latency if you experience underruns/overruns. \
  *default* 50000 (microseconds)

- **BMIX_ALSA_CONF** \
  location of dynamic alsa configuration file: _**must be the same as the value included at the end of the static alsa configuration file**_ \
  *default* /var/lib/bmix/bmix.conf

- **BMIX_IPC_KEY** \
  dmix ipc will use 8 consecutive ipc key numbers starting with this value \
  *default* 30000

- **BMIX_LOOPBACK** \
  to reserve some Loopback substreams for other applications, set the lowest substream to use \
  *default* 0

- **BMIX_DEFAULT_PCM_NAME** \
  name to be used for default bluetooth pcm \
  *default* bluetooth

- **BMIX_ALSALOOP_ARGS** \
  extra arguments to apply to alsaloop \
  *default* --sync=none

#### Systemd unit

For systems using systemd, the bmix daemon can be managed by installing the following unit file as:

`/usr/local/lib/systemd/system/bmix.service`

```
[Unit]
Description=Bluealsa Dmix daemon
Documentation=https://github.com/Arkq/bluez-alsa/wiki/Using-bluealsa-with-dmix
After=bluealsa.service
Wants=bluealsa.service
ConditionPathExists=/proc/asound/Loopback

[Service]
Type=simple
EnvironmentFile=-/etc/default/bmix
ExecStart=/usr/local/bin/bmixd.bash
Restart=on-failure
User=bmix
Group=audio

[Install]
WantedBy=bluetooth.target
```

**Download the file [bmix.service](files/Using-bluealsa-with-dmix/bmix.service)**

To use any of the supported environment variables, create a file
```
/etc/default/bmix
```
with the required enviroment variable definitions, one per line. For example:
```
BMIX_PROFILE=a2dp
BMIX_LATENCY=80000
```
The unit needs to be enabled to start at boot; to do that use:

`sudo systemctl enable bmix.service`

This unit file will cause the bmix daemon to be started as part of the bluetooth system, but only if the snd-aloop Loopback module is loaded.

#### Modules unit

For systems using systemd, it is possible to have the alsa Loopback module loaded at boot by installing the following file as:

`/usr/local/lib/modules-load.d/snd_aloop.conf`

```
# load the alsa Loopback device at boot
snd-aloop
```

If you wish to prevent the Loopback card grabbing card index 0, then also create the file:

`/etc/modprobe.d/snd_aloop.conf`

```
# prevent Loopback module from grabbing card index 0
options snd_aloop index=-2
```

### Usage

The bmix daemon hides all the complexity of setting up the loopback. When a bluetooth playback device is connected, the system creates an ALSA pcm configuration for a dmix interface to the device. The name is generated using the same algorithm as bluealsa uses for the corresponding control device. So, for example, when a device called `Jabra MOVE v2.3.0` is connected using the A2DP profile, then `amixer -D bluealsa scontrols` will show:

```
Simple mixer control 'Jabra MOVE v2.3.0 - A2DP',0
```

and `aplay -L` will show
```
Jabra MOVE v2.3.0 - A2DP
    Jabra MOVE v2.3.0 (A2DP) Bluetooth Audio Playback
```

Sending audio to the bluetooth device is then as simple as:
```
aplay -D 'Jabra MOVE v2.3.0 - A2DP' music.wav
```
Multiple programs can use the pcm at the same time: no other setup by the user is necessary.

Multiple bluetooth playback devices can be connected at the same time, and each will be given its own ALSA configuration entry with its own name.


In addition a pcm called `bluetooth` is configured as an alias for one of the connected bluealsa pcms. It can be used as a default bluetooth device as:
```
aplay -D bluetooth music.wav
```

For "one-shot" applications such as aplay and mpg123, that is sufficient. However, for "long-running" applications that may open and close pcm devices as the application is running, there is one more gotcha in alsa-lib to deal with. Alsa-lib reads its configuration files when the first pcm is opened. This configuration is then cached within the process, and only updated when alsa-lib detects that its main configuration file has changed. By default, alsa-lib does not check any other configuration files. In particular, it does not check for changes to the bmix dynamic sonfiguration. So if an application first opens any pcm when a bluetooth speaker is disconnected, it will not later be able to open a pcm on that bluetooth speaker; its configuration will always say "no such device". Fortunately, there is a workaround: the downside is that it has to be applied to all such applications.

Although the default alsa-lib behaviour is to only check its "main" configuration file for changes, this can be overridden using an environment variable. When an alsa application has the variable `ALSA_CONFIG_PATH` defined in its enviroment, then it will check the files listed in that environment variable _**instead of**_ the default file. So, if this technique is used, it is important to include the default file in the given path list:

`ALSA_CONFIG_PATH=/usr/share/alsa/alsa.conf:/var/lib/bmix/bmix.conf`

This environment variable must be applied to all alsa applications that require to be aware of new bluetooth device connections while they are running. For desktop sessions, include it in the session environment so that all applications launched from the desktop have it set. For system services, set it via the system management script (eg in the systemd unit file, etc).

The daemon will only configure the first 8 bluetooth playback pcms that connect. To introduce a new pcm after that, it is necessary to restart the daemon; this will clear the current substream mapping.

Note that if a bluetooth device disconnects while in use, this does not affect the dmix device that the playing applications are using. So they will continue to play audio even though there is no device there to render it. There does not appear to be any way to force the input Loopback device to close its stream when the output device is closed.

